#!/bin/sh

dump_files_info() {
# $1 - path
	local path=${1:-.}
	echo "dump_files_info ""$@"
	echo "PWD=`pwd`"
	echo "ls -l:"
	find "$path" -exec ls -l \{\} \;
	echo "file:"
	find "$path" -type f -exec file \{\} \;
	echo "ldd:"
	find "$path" -type f -print -exec ldd \{\} \;
}

log_setup() {
# $1 - log file
	LOG_FILE=${1:-/tmp/default_log}
}

log_reset() {
	echo -n > "$LOG_FILE"
}

log_message() {
# $1... - message (optional; if omitted then get message from stdio)
# stdio - message data (ignored if message was passed with argument(s))
	if [ $# -gt 0 ]; then
		echo "$@" >> "$LOG_FILE"
	else
		cat >> "$LOG_FILE"
	fi
}

print_message() {
# $* - messages in a single line
	echo "$*"
	sleep 1 # wait_to_allow_to_see_info_line
}

DISTRO_SUSE=suse
DISTRO_DEBIAN=debian
DISTRO_ALTLINUX=altlinux
DISTRO_FEDORA=fedora
DISTRO_MANDRAKE=mandrake
DISTRO_REDHAT=redhat

identify_distro() {
	log_message "identify_distro" "$@"

	if [ -f /etc/SuSE-release ]; then
		DISTRO=$DISTRO_SUSE

	elif [ -f /etc/debian_version ]; then
		DISTRO=$DISTRO_DEBIAN

	elif [ -f /etc/altlinux-releas ]; then
		DISTRO=$DISTRO_ALTLINUX

	elif [ -f /etc/fedora-release ]; then
		DISTRO=$DISTRO_FEDORA

	elif [ -f /etc/mandrake-release ]; then
		DISTRO=$DISTRO_MANDRAKE

	elif [ -f /etc/redhat-release ]; then
		DISTRO=$DISTRO_REDHAT
	fi

	echo $DISTRO
}

check_packages_dpkg() {
# $1... - package(s)
	local MISSING
	log_message "check_packages_dpkg" "$@" $#
	log_message "all packages:"; dpkg --list 2>&1 | log_message

	MISSING=
	for PKG_NAME in $@; do
		dpkg-query --status $PKG_NAME | grep "Status:" | grep -e "\ \<installed\>" >/dev/null
		if [ $? -ne 0 ]; then
			MISSING="$MISSING $PKG_NAME"
		fi
	done
	echo "$MISSING"
}

check_packages_rpm() {
# $1... - package(s)
	local MISSING
	log_message "check_packages_rpm" "$@" $#
	log_message "all packages:"; rpm -qa 2>&1 | log_message

	MISSING=
	for PKG_NAME in $@; do
		rpm -q $PKG_NAME > /dev/null
		if [ $? -ne 0 ]; then
			MISSING="$MISSING $PKG_NAME"
		fi
	done
	echo "$MISSING"
}

check_packages() {
# $1 - distro
# $2... - package(s)
	local DISTRO=$1
	log_message "check_packages" "$@" $#

	shift

	case $DISTRO in
		$DISTRO_SUSE|$DISTRO_REDHAT|$DISTRO_FEDORA|$DISTRO_ALTLINUX|$DISTRO_MANDRAKE)
			log_message "RPM package system"
			check_packages_rpm "$@"
			;;
		$DISTRO_DEBIAN)
			log_message "DPKG package system"
			check_packages_dpkg "$@"
			;;
		*)
			echo "Unsupported distribution" >&2
			log_message "Unsupported distribution <$DISTRO>"
			return 1
			;;
	esac
}

install_packages() {
# $1 - distro
# $2... - package(s)
	local DISTRO=$1
	local install_packages_script=/tmp/install_packages.sh
	local install_packages_log=/tmp/install_packages_log
	log_message "install_packages" "$@" $#

	shift

	log_message "id: `id`"

	COMMAND=
	case $DISTRO in
		suse)
#			if which zypper; then
#				COMMAND="zypper install"
#			else
				COMMAND="yast2 -i"
#			fi
			;;
		debian)
			COMMAND="apt-get install"
			;;
		altlinux)
			COMMAND="apt-get install"
			;;
		fedora | redhat)
			COMMAND="yum install"
			;;
		mandrake)
			COMMAND="urpmi"
			;;
		*)
			echo "Unknown distribution '$DISTRO'"
			return 1
			;;
	esac

	log_message "COMMAND=$COMMAND"
	log_message "DISTRO=$DISTRO"

	COMMAND=$COMMAND\ "$@"
	log_message "COMMAND=$COMMAND"

	if [ "x$DISPLAY" = 'x' ]; then
		log_message "1:"
		$COMMAND 2>&1 | tee "$install_packages_log"
	elif [ "x$DISPLAY" = "x" ]; then
		log_message "1_alt:"
		$COMMAND 2>&1 | tee "$install_packages_log"
	elif [ -z "$DISPLAY" ]; then
		log_message "1_alt2:"
		$COMMAND 2>&1 | tee "$install_packages_log"
	else
		log_message "2:"
		if [ "x$DISTRO" = 'xsuse' ] ; then #&&  ! which zypper > /dev/null; then
			log_message "21:"
			$COMMAND 2>&1 | tee "$install_packages_log"
		else
			log_message "22:"
			if which xfce4-terminal; then
				log_message "2201:"
				xfce4-terminal -e "$COMMAND" 2>&1 | log_message
			elif which gnome-terminal; then
				log_message "2202:"
				echo "$COMMAND 2>&1 | tee $install_packages_log" > $install_packages_script
				gnome-terminal --disable-factory -x sh $install_packages_script 2>&1 | log_message
			elif which konsole; then
				log_message "2203:"
				konsole -e $COMMAND 2>&1 | log_message
				while ps aux | grep "konsole -e $COMMAND" | grep -v grep
				do
					log_message "Waiting for child to stop..."
					echo "Waiting for child to stop..."
					sleep 5
				done
			elif which Eterm; then
				log_message "2204:"
				Eterm -e $COMMAND 2>&1 | log_message
			elif which rxvt; then
				log_message "2205:"
				rxvt -e $COMMAND 2>&1 | log_message
			elif which aterm; then
				log_message "2206:"
				aterm -e $COMMAND 2>&1 | log_message
			elif which lxterminal; then
				log_message "2207:"
				lxterminal -e $COMMAND 2>&1 | log_message
			elif which xterm; then
				log_message "2208:"
				xterm -e $COMMAND 2>&1 | log_message
			elif [ "x$COLORTERM" != "x" ]; then
				log_message "2209:"
				$COLORTERM -e "$COMMAND" 2>&1 | log_message
			elif [ "x$TERM" != "x" ]; then
				log_message "2210:"
				$TERM -e "$COMMAND" 2>&1 | log_message
			else
				log_message "2211:"
				log_message "Can not find terminal in the system! Aborting."
				echo "Can not find terminal in the system! Aborting."
				return 1
			fi
		fi
	fi

	log_message "install_packages finished with code <$?>"
}

required_packages_handler_default() {
	log_message "packages_handler_default" "$@"
}

install_required() {
# $1 - required packages handler (function name; called to get list of required packages)
	local required_packages_handler=${1:-required_packages_handler_default}
	local DISTRO=`identify_distro`
	local REQUIRED
	local MISSING
	log_message "install_required" "$@"

	print_message "Installing required packages..."
	
	log_message "Target distribution: $DISTRO"

	eval REQUIRED=\$\($required_packages_handler \"\$DISTRO\"\)
	log_message "Required packages: <$REQUIRED>"

	log_message "Packages before install:"
	MISSING=`check_packages "$DISTRO" "$REQUIRED"`
	log_message "MISSING PACKAGES BEFORE: <$MISSING>"

	if [ "x$MISSING" = x ]; then 
		log_message "All the packages required are already installed!"
		echo "All the packages required are already installed!"
		return 0
	fi

	log_message "Packages to install: <$MISSING>"
	echo MISSING PACKAGES: $MISSING

	install_packages "$DISTRO" $MISSING

	log_message "Packages after install:"
	MISSING=`check_packages "$DISTRO" "$REQUIRED"`
	log_message "MISSING PACKAGES AFTER: <$MISSING>"

	log_message "check_and_install finished!"

	test -z "$MISSING"
}


